@@ -20,7 +20,9 @@ import importlib
 import sphobjinv
 import functools
 import pylit
+from sphinx import __version__ as sphinx_version
 from sphinx.ext.napoleon.docstring import NumpyDocstring
+from packaging.version import Version
 
 # apidoc
 sys.path.insert(0, os.path.abspath('.'))
@@ -32,6 +34,7 @@ _today = datetime.datetime.now()
 # https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
 
 package = 'slepc4py'
+project = 'slepc4py'
 
 docdir = os.path.abspath(os.path.dirname(__file__))
 topdir = os.path.abspath(os.path.join(docdir, *[os.path.pardir] * 2))
@@ -117,6 +120,12 @@ autosummary_context = {
     'autotype': {},
 }
 
+suppress_warnings = []
+if Version(sphinx_version) >= Version(
+    '7.4'
+):  # https://github.com/sphinx-doc/sphinx/issues/12589
+    suppress_warnings.append('autosummary.import_cycle')
+
 # Links depends on the actual branch -> release or main
 www = f'https://gitlab.com/slepc/slepc/-/tree/{get_doc_branch()}'
 extlinks = {'sources': (f'{www}/src/binding/slepc4py/src/%s', '%s')}
@@ -256,14 +265,14 @@ def _setup_autodoc(app):
 
     #
 
-    def stringify_annotation(annotation, mode='fully-qualified-except-typing'):
+    def stringify_annotation(annotation, *p, **kw):
         qualname = getattr(annotation, '__qualname__', '')
         module = getattr(annotation, '__module__', '')
         args = getattr(annotation, '__args__', None)
         if module == 'builtins' and qualname and args is not None:
-            args = ', '.join(stringify_annotation(a, mode) for a in args)
+            args = ', '.join(stringify_annotation(a, *p, **kw) for a in args)
             return f'{qualname}[{args}]'
-        return stringify_annotation_orig(annotation, mode)
+        return stringify_annotation_orig(annotation, *p, **kw)
 
     try:
         stringify_annotation_orig = typing.stringify_annotation
@@ -277,6 +286,8 @@ def _setup_autodoc(app):
         typing.stringify = stringify_annotation
         autodoc.stringify_typehint = stringify_annotation
 
+    inspect.TypeAliasForwardRef.__repr__ = lambda self: self.name
+
     #
 
     class ClassDocumenterMixin:
@@ -321,7 +332,9 @@ def _monkey_patch_returns():
     @functools.wraps(NumpyDocstring._parse_returns_section)
     def wrapper(*args, **kwargs):
         out = _parse_returns_section(*args, **kwargs)
-        return [line.replace(':class:', ':any:') for line in out]
+        for role in (':py:class:', ':class:'):
+            out = [line.replace(role, ':any:') for line in out]
+        return out
 
     NumpyDocstring._parse_returns_section = wrapper
 
@@ -337,7 +350,9 @@ def _monkey_patch_see_also():
     @functools.wraps(NumpyDocstring._parse_numpydoc_see_also_section)
     def wrapper(*args, **kwargs):
         out = _parse_numpydoc_see_also_section(*args, **kwargs)
-        return [line.replace(':obj:', ':any:') for line in out]
+        for role in (':py:obj:', ':obj:'):
+            out = [line.replace(role, ':any:') for line in out]
+        return out
 
     NumpyDocstring._parse_numpydoc_see_also_section = wrapper
 
@@ -377,14 +392,17 @@ def _process_demos(*demos):
         html_static_path.append(demo_copy_name)
     with open(os.path.join('demo', 'demo.rst'), 'w') as demofile:
         demofile.write("""
+.. _demos:
+
 slepc4py demos
 ==============
 
 .. toctree::
+   :maxdepth: 1
 
 """)
         for demo in demos:
-            demofile.write('    ' + os.path.splitext(demo)[0] + '\n')
+            demofile.write('   ' + os.path.splitext(demo)[0] + '\n')
         demofile.write('\n')
 
 
@@ -392,7 +410,22 @@ html_static_path = ['_static']
 html_css_files = [ # relative to the html_static_path
                   'css/slepc.css',
                   ]
-_process_demos('ex1.py')
+_process_demos(
+                'ex1.py',
+                'ex2.py',
+                'ex3.py',
+                'ex4.py',
+                'ex5.py',
+                'ex6.py',
+                'ex7.py',
+                'ex8.py',
+                'ex9.py',
+                'ex10.py',
+                'ex11.py',
+                'ex12.py',
+                'ex13.py',
+                'ex14.py',
+                )
 
 
 def setup(app):
@@ -483,9 +516,12 @@ html_theme = 'pydata_sphinx_theme'
 
 html_theme_options = {
     'navigation_with_keys': True,
-    "footer_end": ["theme-version", "last-updated"],
+    'footer_end': ['theme-version', 'last-updated'],
+    'header_links_before_dropdown': 10, # before "more"
 }
-git_describe_version = subprocess.check_output(['git', 'describe', '--always']).strip().decode('utf-8') # noqa: S603, S607
+git_describe_version = (
+    subprocess.check_output(['git', 'describe', '--always']).strip().decode('utf-8')  # noqa: S603, S607
+)
 html_last_updated_fmt = r'%Y-%m-%dT%H:%M:%S%z (' + git_describe_version + ')'
 
 # -- Options for HTMLHelp output ------------------------------------------
